print("import: 'flask_admin'")
import flask_admin

print("import: 'flask_admin.contrib'")
import flask_admin.contrib

print("import: 'flask_admin.contrib.fileadmin'")
import flask_admin.contrib.fileadmin

print("import: 'flask_admin.form'")
import flask_admin.form

print("import: 'flask_admin.model'")
import flask_admin.model

print("import: 'flask_admin.translations'")
import flask_admin.translations

